@@ -600,9 +600,12 @@ func TestCmd_validate(t *testing.T) {
 		{"only sync", Cmd{Sync: SyncInternal{Source: "source", Dest: "dest"}}, ""},
 		{"only msync", Cmd{MSync: []SyncInternal{{Source: "source", Dest: "dest"}}}, ""},
 		{"only wait", Cmd{Wait: WaitInternal{Command: "command"}}, ""},
+		{"only line", Cmd{Line: LineInternal{File: "/etc/bashrc", Match: "^PS1=", Delete: true}}, ""},
+		{"line without operation", Cmd{Line: LineInternal{File: "/etc/bashrc", Match: "^PS1="}},
+			"one of [script, copy, mcopy, delete, mdelete, sync, msync, wait, line, echo] must be set"},
 		{"multiple fields set", Cmd{Script: "example_script", Copy: CopyInternal{Source: "source", Dest: "dest"}},
 			"only one of [script, copy] is allowed"},
-		{"nothing set", Cmd{}, "one of [script, copy, mcopy, delete, mdelete, sync, msync, wait, echo] must be set"},
+		{"nothing set", Cmd{}, "one of [script, copy, mcopy, delete, mdelete, sync, msync, wait, line, echo] must be set"},
 		{"script with register", Cmd{Script: "example_script", Register: []string{"a", "b"}}, ""},
 		{"unexpected register", Cmd{Copy: CopyInternal{Source: "source", Dest: "dest"}, Register: []string{"a", "b"}},
 			"register is only allowed with script command"},
